/****************************************************************************
         Amnuts version 2.3.0 - Copyright (C) Andrew Collington, 2003
                      Last update: 2003-08-04

                              amnuts@talker.com
                          http://amnuts.talker.com/

                                   based on

   NUTS version 3.3.3 (Triple Three :) - Copyright (C) Neil Robertson 1996
 ***************************************************************************/

#ifndef AMNUTS_DEFINES_H
#define AMNUTS_DEFINES_H

#include <sys/types.h>
#include <sys/wait.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/utsname.h>
#include <sys/select.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
#include <signal.h>
#include <time.h>
#include <errno.h>
#include <fcntl.h>
#include <dirent.h>
#include <stddef.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <strings.h>


/*
 * version information
 * Use USERVER for whatever you want, but please don't change MAMNUTSVER and NUTSVER
 */
#define MAMNUTSVER   "1.8.1"
#define NUTSVER     "3.3.3"
#define USERVER     "0.3"
#define TALKER_NAME "Your Talker's Name Here"

/* general directories */
#define DATAFILES  "datafiles"
#define HELPFILES  "helpfiles"
#define MAILSPOOL  "mailspool"
#define MISCFILES  "miscfiles"
#define PICTFILES  "pictfiles"
#define MOTDFILES  "motds"
#define DUMPFILES  "dumpfiles"
#define TEXTFILES  "textfiles"
#define ADMINFILES "adminfiles"
#define LOGFILES   "logfiles"

/* user directories */
#define USERFILES     "userfiles"
#define USERMAILS     "mail"
#define USERPROFILES  "profiles"
#define USERHISTORYS  "historys"
#define USERCOMMANDS  "xgcoms"
#define USERMACROS    "macros"
#define USERROOMS     "rooms"
#define USERREMINDERS "reminders"
#define USERFLAGGED   "flagged"

/* seemless reboot */
#define REBOOTING_DIR           "reboot"
#define USER_LIST_FILE          "reboot/_ulist"
#define TALKER_SYSINFO_FILE     "reboot/_sysinfo"
#define CHILDS_PID_FILE         "reboot/_child_pid"
#define ROOM_LIST_FILE          "reboot/_rlist"
#define LAST_USERS_FILE         "reboot/_last"

/* files */
#define CONFIGFILE   "config"
#define NEWSFILE     "newsfile"
#define SITEBAN      "siteban"
#define USERBAN      "userban"
#define NEWBAN       "newban"
#define SUGBOARD     "suggestions"
#define RULESFILE    "rules"
#define WIZRULESFILE "wizrules"
#define HANGDICT     "hangman_words"
#define SHOWFILES    "showfiles"

/* system logs */
#define LAST_CMD   "last_command"
#define MAINSYSLOG "syslog"
#define REQSYSLOG  "reqlog"
#define NETSYSLOG  "netlog"
#define ERRSYSLOG  "errlog"
#define SYSLOG BIT(0)
#define REQLOG BIT(1)
#define NETLOG BIT(2)
#define ERRLOG BIT(3)

/* general defines */
#define MAXADDR 16              /* XXX: Use NI_MAXHOST, INET_ADDRSTRLEN, INET6_ADDRSTRLEN */
#define MAXHOST 1025            /* XXX: Use NI_MAXHOST */
#define MAXSERV 32              /* XXX: Use NI_MAXSERV */
#define OUT_BUFF_SIZE 1000
#define MAX_WORDS 10
#define WORD_LEN 80
#define ARR_SIZE 1000
#define MAX_LINES 15
#define REVIEW_LINES 45
#define REVTELL_LINES 45
#define REVIEW_LEN 400
#define BUFSIZE 1000
#define ROOM_NAME_LEN 20
#define ROOM_LABEL_LEN 5
#define LASTLOGON_NUM 15
#define LOGIN_FLOOD_CNT 20
#define UFILE_WORDS 16          /* must be at least 1 longer than max words in the _options */
#define UFILE_WORD_LEN 255
#define RECORD 1
#define NORECORD 0
#define LANGUAGE "en"


/* netlink defines */
#define VERIFY_LEN 20
#ifdef NETLINKS
#define SERV_NAME_LEN 80
#endif

/* user defines */
#define USER_NAME_MIN 2  
#define USER_NAME_LEN 16
#define RECAP_NAME_LEN (USER_NAME_LEN*4 + 3)
#define USER_DESC_LEN (40 - USER_NAME_LEN)
#define AFK_MESG_LEN 60
#define PHRASE_LEN 40
#define PASS_MIN 3              /* probably bad to allow extremely short passwords */
#define PASS_LEN 13             /* crypt() generates this many plus nul */
#define ROOM_DESC_LEN (MAX_LINES*81)    /* MAX_LINES lines of 80 chars each + MAX_LINES nl */
#define TOPIC_LEN 60
#define ICQ_LEN 20
#define NEUTER 0
#define MALE   1
#define FEMALE 2
#define NEWBIE_EXPIRES 20       /* days */
#define USER_EXPIRES   40       /* days */
#define SCREEN_WRAP 80          /* how many characters to wrap to */
#define MAX_COPIES 6            /* of smail */
#define MACRO_LEN 65
#define MAX_XCOMS 10
#define MAX_GCOMS 10
#define MAX_PAGES 1000          /* should be enough! */
#define MAX_REMINDERS 30
#define REMINDER_LEN  70

/* rooms */
#define MAX_LINKS 20
#define PRIVATE  BIT(0)
#define FIXED    BIT(1)
#define PERSONAL BIT(2)
#define PERSONAL_ROOMNAME_LEN 80

/* logon prompt stuff */
#define LOGIN_ATTEMPTS 3

#ifdef IDENTD
/* ident daemon stuff */
#define IDENTUSER "Identduser"
#endif

/*
 * some macros that are used in the code
 * these are for grammer
 */
#define PLTEXT_S(n) ((1==(n))?"":"s")
#define PLTEXT_ES(n) ((1==(n))?"":"es")
#define PLTEXT_IS(n) ((1==(n))?"is":"are")
#define PLTEXT_WAS(n) ((1==(n))?"was":"were")
#define SIZEOF(table) ((sizeof (table))/(sizeof *(table)))
/* these are for bit manipulation */
#define BIT(pos) (1L<<(pos))

/* money code */
#define DEFAULT_MONEY 1000
#define DEFAULT_BANK 3000
#define MAX_DONATION 5000
#define CREDITS_PER_HOUR 20
#define MIN_CREDIT_UPDATE_LEVEL SUPER

/* flags */
#define GL_PRESERVE_ORIG BIT(20)
#define fufROOMKEY BIT(0)
#define fufIGNORE  BIT(1)
#define fufFRIEND  BIT(2)
#define rbfTELL    BIT(0)
#define rbfEDIT    BIT(1)
#define rbfAFK     BIT(2)

#endif
